@@ -1,14 +1,17 @@
+import sys
 import datetime
-from kraken_thing import json_xp as json
-from kraken_thing.kraken_class_observation import Observation
 import uuid
 from urllib.parse import urlparse
-from kraken_schema_org import kraken_schema_org as k
-from kraken_thing import normalize_id
-import sys
-import kraken_datatype as dt 
-from kraken_schema_org import kraken_schema_org as k
-from kraken_thing import normalize_value as nv
+
+from kraken_schema_org import kraken_schema_org as kraken_schema
+from kraken_thing.libraries import json_xp as json
+from kraken_thing.libraries import normalize_id
+from kraken_thing.libraries import normalize_value as nv
+from kraken_thing.libraries import key_mapping
+from kraken_thing.libraries import dict_helper
+
+from kraken_thing.kraken_class_observations import Observations
+
 
 
 class Thing:
@@ -16,11 +19,34 @@ class Thing:
     def __init__(self, record_type = None, record_id = None):
         """
         A class to represent a schema.org Thing
+
+        methods:
+            get(key):
+                retrieve all values for given key
+            get_best(key):
+                return single best value for given key
+            set(key, value): 
+                set a new value for a given key
+            load(record):
+                load all values in record
+            dump():
+                output a records with all values
+        
         """
         
-        self.db = []
-        self.things = None     # Reference to things collection
+        self._observations = Observations()   # Gets replaced with common if in Things
+                   
+        self._things = None     # Reference to things collection
 
+        # Metadata
+        self._source_observations_id = []
+        self._object = None
+        self._agent = None
+        self._instrument = None
+        self._credibility = None
+        self._date = None
+        
+        
         # Load record if record instead of record_type
         if record_type and isinstance(record_type, dict):
             self.load(record_type)
@@ -30,13 +56,12 @@ class Thing:
 
             
         # Add id if none
-        if record_type and not record_id:
+        if not self.record_id:
             self.record_id = str(uuid.uuid4())
 
     
     def __str__(self):
         
-       
         return str(self.summary)
 
     def __repr__(self):
@@ -47,7 +72,7 @@ class Thing:
         """
         """
 
-        if not isinstance(other, Thing):
+        if not issubclass(type(other), Thing):
             return False
         
         if self.record_id and self.record_id == other.record_id and self.record_type and self.record_type == other.record_type:
@@ -62,20 +87,40 @@ class Thing:
         new_t = Thing()
 
         #Load self record
-        new_t.set_observations(self.get_observations())
+        new_t._observations.set(self._observations.get())
 
         # Load other record
-        new_t.set_observations(other.get_observations())
+        new_t._observations.set(other._observations.get())
 
         return new_t
 
+    def __setattr__(self, attr, value):
+        '''
+        '''
+        if attr.startswith('_') or attr in self.__dict__.keys():
+            self.__dict__[attr] = value
+        else:
+            attr = key_mapping.get(attr, attr)
+            self.set(attr, value)
+
+    def __getattr__(self, attr):
+        '''
+        '''
+        if attr.startswith('_') or attr in self.__dict__.keys():
+            return self.__dict__.get(attr, None)
+            
+        else:
+            attr = key_mapping.get(attr, attr)
+            return self.get_best(attr)
+        
+    
     def merge(self, other):
         '''Add obs from other in self
         '''
                 
         # Copy observations
-        observations = other.get_observations()
-        self.set_observations(observations)
+        observations = other._observations.get()
+        self._observations.set(observations)
             
         return
 
@@ -89,7 +134,7 @@ class Thing:
         content.append('@id: ' + str(self.record_id) )
         content.append(40 * '-' )
 
-        for i in self.get_observations():
+        for i in self._observations.get():
             content.append(str(i.key) + ': ' + ((20 - len(i.key)) * ' ' )+ str(i.value) )
         return '\n'.join(content)
 
@@ -100,7 +145,7 @@ class Thing:
         '''
         content = []
         base_obs = []
-        for i in self.get_observations():
+        for i in self._observations.get():
             if not i.source_observations_id:
                 base_obs.append(i)
 
@@ -123,7 +168,7 @@ class Thing:
         '''For each, shows where it comes from
         '''
         content = []
-        obs = self.get_observations()
+        obs = self._observations.get()
 
         def recurse_relations_childs(obs, level=0):
             content = []
@@ -146,23 +191,31 @@ class Thing:
         """Create a new observation with key value. 
         """
 
+        metadata = metadata if metadata else self.metadata
+        
         # Error handling
         if not value:
             return 
-
-        
+            
         # Handle lists (in reverse so it conserves proper order)
         if isinstance(value, list) and not isinstance(value, str):
             for i in reversed(value):
                 self.set(key, i, metadata)
             return
 
+        # Handle thing as value
+        if isinstance(value, dict) and '@type' in value.keys():
+            new_value = Thing()
+            new_value.load(value)
+            new_value.metadata = metadata
+            value = new_value            
 
+            
         # Initialize 
         observations = []
         
         # Get original observation (as is key value)
-        o = self.new_observation(key, value, metadata)
+        o = self._observations.new(key, value, metadata)
         if o:
             observations.append(o)
 
@@ -177,7 +230,7 @@ class Thing:
             observations.append(o)
         
         # Add obs to db
-        self.set_observations(observations)
+        self._observations.set(observations)
         
 
     def get(self, key, include_non_valid = True):
@@ -194,34 +247,24 @@ class Thing:
         if not key.startswith('@') and not key.startswith('schema:'):
             key = 'schema:' + key
 
-        obs = []
-        for i in self.db:
-            if i.key == key:
-                obs.append(i)
+        obs = self._observations.get_by_key(key, include_non_valid)
 
-        if not obs or len(obs) == 0:
-            return []
-
-        # Sort observations
-        values = []
-        for i in sorted(obs, reverse=True):
-            if include_non_valid or (i.valid_key and i.valid_value):
+        values = []                            
+        for i in obs:
+            if i.value not in values:
                 values.append(i.value)
-            
-            
+
         return values
+        
 
     def get_best(self, key):
         '''Returns best value
         '''
         
-        value = self.get(key, False)
+        obs = self._observations.get_by_key(key)
 
-        if not value:
-            value = self.get(key, True)
-
-        if value and len(value) > 0:
-            return value[0]
+        if obs and len(obs)> 0:
+            return obs[0].value
         else:
             return None
 
@@ -273,7 +316,7 @@ class Thing:
                 values = [values]
 
             for v in values:
-                if isinstance(v, Thing):
+                if issubclass(type(v), Thing):
                     v = v.dump()
 
                 if v not in record[k]:
@@ -285,14 +328,9 @@ class Thing:
         record['@id'] = self.record_id
         
         # Remove lists and empty values
-        new_record = {}
-        for k, v in record.items():
-            if v and len(v) == 1:
-                new_record[k] = v[0]
-            elif v and len(v) > 1:
-                new_record[k] = v
+        record = dict_helper.clean(record)
         
-        return new_record
+        return record
 
     @property
     def json(self):
@@ -307,118 +345,14 @@ class Thing:
         record = json.loads(value)
         self.load(record)
         
-
-    """Observations
-    """
-
-    def set_observations(self, observation):
-        '''
-        '''
-        if isinstance(observation, list):
-            for i in observation:
-                self.set_observations(i)
-            return
-            
-        if observation not in self.db:
-            self.db.append(observation)
-        
-        # Add parents and childs
-        for i in observation.source_observations_id:
-            source_obs = self.get_observation(i)
-
-            if source_obs not in observation.source_observations:
-                observation.source_observations.append(source_obs)
-            
-            if observation not in source_obs.child_observations:
-                source_obs.child_observations.append(observation)
-
-    
-    def get_observation(self, observation_id):
-        '''Return an observation by id
-        '''
-        for i in self.db:
-            if i.observation_id == observation_id:
-                return i
-        return None
-    
-    def get_observations(self, key = None):
-        '''Returns all observations sorted best to worst by group key
-        '''
-
-        obs = []
-
-        # Get obs by key
-        for key1 in self.keys:
-            key_obs = []
-            for i in self.db:
-                if key and i.key == key and i.key == key1:
-                    key_obs.append(i)
-                elif not key and i.key == key1:
-                    key_obs.append(i)
-
-            key_obs = sorted(key_obs, reverse=True)
-            obs += key_obs
-        
-        return obs
-
-    def get_best_observation(self, key = None):
-        '''
-        '''
-        obs = self.get_best_observations(key)
-
-        if len(obs) > 0:
-            return obs[0]
-        else:
-            return None
-        
-    
-    def load_observations(self, observations):
-        '''Load observations in list of dict format
-        '''
-        if not isinstance(observations, list):
-            observations = [observations]
-
-        for i in observations:
-            o = Observation(self.record_type, self.record_id)
-            o.load(i)
-            self.set_observations(o)
-            
-        return
-
-    def dump_observations(self):
-        '''Returns list of obs in dict format
-        '''
-        obs = []
-        for i in self.db:
-            obs.append(i.dump())
-
-        return obs
-
-    def new_observation(self, key, value, metadata = None):
-        '''
-        '''
-        # Convert to thing if record
-        if isinstance(value, dict) and '@type' in value.keys():
-            new_v = Thing()
-            new_v.load(value)
-            value = new_v
-            
-        # Record original value
-        observation = Observation(key, value)
-
-        # Load metadata
-        observation.metadata = metadata
-
-        return observation
-
     
     def normalize_observation(self, observation):
         '''Normalize key and value. If both different, create new observation
         '''
 
         # Get normalized key value
-        normalized_record_type = k.normalize_type(self.record_type)
-        normalized_key = k.normalize_key(observation.key) if not observation.key.startswith('@') else observation.key
+        normalized_record_type = kraken_schema.normalize_type(self.record_type)
+        normalized_key = kraken_schema.normalize_key(observation.key) if not observation.key.startswith('@') else observation.key
         normalized_value = nv.normalize_value(normalized_record_type, observation.key, observation.value)   
 
 
@@ -429,7 +363,7 @@ class Thing:
             observation.valid_value = True
 
         # Address if value is thing
-        if isinstance(observation.value, Thing):
+        if issubclass(type(observation.value), Thing):
             observation.valid_value = True
 
         # Skip if already normalized
@@ -484,58 +418,7 @@ class Thing:
     
     """Properties
     """
-    @property
-    def record_type(self):
-        '''
-        '''
-        record_type = self.get_best('@type')
-        return record_type
-
-    @record_type.setter
-    def record_type(self, value):
-        '''
-        '''
-        
-        self.set('@type', value)
-
-        
-
-    @property
-    def record_id(self):
-        '''
-        '''
-
-        record_id = self.get_best('@id')
-        return record_id
-        
-
-    @record_id.setter
-    def record_id(self, value):
-        '''
-        '''
     
-        if isinstance(value, list) and len(value) == 0:
-            return
-        if isinstance(value, list) and len(value) == 1:
-            value=value[0]
-        if value is not None and not isinstance(value, str):
-            return
-
-        self.set('@id', value)
-            
-
-    @property
-    def record_ids(self):
-        '''Return list of current and original record_ids
-        '''
-        record_ids = []
-        record_ids.append(self.record_id)
-        for i in self._original_record_ids:
-            if i not in record_ids:
-                record_ids.append(i)
-        return record_ids
-    
-
 
     @property
     def record_ref(self):
@@ -549,6 +432,8 @@ class Thing:
     def record_ref(self, value):
         self.load(value)
 
+    
+    
     @property
     def keys(self, valid = False):
         '''
@@ -556,7 +441,7 @@ class Thing:
 
         # Get keys
         keys = []
-        for i in self.db:
+        for i in self._observations.get():
             if i.key not in keys:
                 if valid and not i.valid_key:
                     continue
@@ -567,22 +452,7 @@ class Thing:
         return keys
         
     
-    @property
-    def name(self):
-        return self.get_best('schema:name')
-
-    @name.setter
-    def name(self, value):
-        self.set('schema:name', value)
-
-    @property
-    def url(self):
-        return self.get_best('schema:url')
-
-    @url.setter
-    def url(self, value):
-        self.set('schema:url', value)
-
+    
     @property
     def url_domain(self):
         '''
@@ -597,14 +467,34 @@ class Thing:
         return domain
 
 
+    @property
+    def metadata(self):
+        '''
+        '''
+        record = {
+            '_source_observations_id': self._source_observations_id,
+            '_agent': self._agent,
+            '_instrument': self._instrument,
+            '_credibility': self._credibility,
+            '_date': self._date
+        }
+        return record
+
+    @metadata.setter
+    def metadata(self, record):
+        '''
+        '''
+        for k, v in record.items():
+            setattr(self, k, v)
+        return
     
     '''Methods
     '''
     def update_record_ref(self, old_record_type, old_record_id, new_record_type, new_record_id):
         '''Updates all value with new record_ref
         '''
-        for i in self.get_observations():
-            if isinstance(i.value, Thing):
+        for i in self._observations.get():
+            if issubclass(type(i.value), Thing):
                 if i.original_value.record_type == old_record_type and i.original_value.record_id == old_record_id:
                     i.original_value.record_type = new_record_type
                     i.original_value.record_id = new_record_id
@@ -681,7 +571,7 @@ class Thing:
         size = 0
         size += sys.getsizeof(self) / 1024
 
-        for i in self.db:
+        for i in self._db:
             size += i.memory_size
         
         return round(size, 2)
\ No newline at end of file
